package ch.epfl.tpp.sol;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

public class RandomDocument implements Document {
    private final int wordCount;
    private final long seed;
    private final List<String> unmodifiableWords;
    private final static char[] VOWELS = new char[] { 'a', 'e', 'i', 'o', 'u', 'y' };
    private final static char[] CONSONANTS = new char[] { 'b', 'c', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'm', 'n', 'p', 'q', 'r', 's', 't', 'v', 'w', 'x', 'z' };
    
    public RandomDocument(int wordCount, long seed) {
        Random rng = new Random(seed);
        ArrayList<String> words = new ArrayList<>(wordCount);
        for (int i = 0; i < wordCount; ++i) {
            final int wordLen = 3 + rng.nextInt(3);
            StringBuilder b = new StringBuilder(wordLen);
            for (int j = 0; j < wordLen; ++j) {
                if ((j & 1) == 0)
                    b.append(VOWELS[rng.nextInt(VOWELS.length)]);
                else
                    b.append(CONSONANTS[rng.nextInt(CONSONANTS.length)]);                    
            }
            words.add(b.toString());
        }

        this.wordCount = wordCount;
        this.seed = seed;
        this.unmodifiableWords = Collections.unmodifiableList(words);
    }
    
    @Override
    public Iterator<String> wordsIterator() {
        return unmodifiableWords.iterator();
    }

    @Override
    public int compareTo(Document thatD) {
        // FIXME: allow comparison with other kinds of documents
        RandomDocument that = (RandomDocument)thatD;
        int wcComp = Integer.compare(this.wordCount, that.wordCount);
        return wcComp != 0 ? wcComp : Long.compare(this.seed, that.seed);
    }
}
